import random
import string
from passlib.hash import bcrypt_sha256

from django.db import models
from allianceauth.services.hooks import NameFormatter
import logging

logging.getLogger(__name__)


class MumbleManager(models.Manager):

    @staticmethod
    def get_display_name(user):
        from .auth_hooks import MumbleService
        return NameFormatter(MumbleService(), user).format_name()

    @staticmethod
    def get_username(user):
        return user.profile.main_character.character_name  # main character as the user.username may be incorect

    @staticmethod
    def sanitise_username(username):
        return username.replace(" ", "_")

    @staticmethod
    def generate_random_pass():
        return ''.join([random.choice(string.ascii_letters + string.digits) for n in range(16)])

    @staticmethod
    def gen_pwhash(password):
        return bcrypt_sha256.encrypt(password.encode('utf-8'))

    def create(self, user):
        try:
            username = self.get_username(user)
            logger.debug(f"Creating mumble user with username {username}")
            username_clean = self.sanitise_username(username)
            display_name = self.get_display_name(user)
            password = self.generate_random_pass()
            pwhash = self.gen_pwhash(password)
            logger.debug("Proceeding with mumble user creation: clean username {}, pwhash starts with {}".format(
                username_clean, pwhash[0:5]))
            logger.info(f"Creating mumble user {username_clean}")

            result = super().create(
                user=user,
                username=username_clean,
                pwhash=pwhash,
                hashfn=self.HashFunction.SHA256,
                display_name=display_name)
            result.update_groups()
            result.credentials.update({'username': result.username, 'password': password})
            return result
        except AttributeError:  # No Main or similar errors
            return False
        return False

    def user_exists(self, username):
        return self.filter(username=username).exists()
